﻿using System.Collections.Generic;
using System.Linq;

namespace VisuIT.ASLL.Example
{
	public class Reader
	{
		private const string DataPath = "/VisuIT/Data/";

		public Reader()
		{
			ASRoot.SetLicenseKey("EHB63-D3M49-E4028-2P7FK-L7T82");
		}

		/// <summary>
		/// Loads an autosar model from the given file.
		/// </summary>
		public AUTOSAR LoadAutosar(string filename)
		{
			return ASRoot.LoadFromFile(filename);
		}

		/// <summary>
		/// Does some analysis on the given autosar model. Assumes a certain structure of the
		/// autosar model.
		/// </summary>
		public void AnalyseAutosar(AUTOSAR autosar)
		{
			AR_PACKAGE rootPackage = autosar.AR_PACKAGES.AR_PACKAGE.Single(x => x.SHORT_NAME.Value == "VisuIT");

			// Get all sender receiver interface references of the ports of application software
			// components.
			IEnumerable<string> allSrInterfaceRefs = rootPackage
				.AR_PACKAGES.AR_PACKAGE.Single(x => x.SHORT_NAME.Value == "Components")
				.ELEMENTS.Choice.OfType<APPLICATION_SW_COMPONENT_TYPE>()
				.SelectMany(app => GetAllSrInterfaceRefs(app));

			// Get all sender receiver interfaces from the data package.
			IEnumerable<SENDER_RECEIVER_INTERFACE> allSrInterfaces = rootPackage
				.AR_PACKAGES.AR_PACKAGE.Single(x => x.SHORT_NAME.Value == "Data")
				.ELEMENTS.Choice.OfType<SENDER_RECEIVER_INTERFACE>();

			// Get all sender receiver interfaces referenced by the ports of application software
			// components.
			IEnumerable<SENDER_RECEIVER_INTERFACE> referencedSrInterfaces =
				from senderReceiver in allSrInterfaces
				join reference in allSrInterfaceRefs on senderReceiver.SHORT_NAME.Value equals GetNameFromDataRef(reference)
				select senderReceiver;

			// Get a distinct list of the format strings of the referenced sender receiver
			// interfaces (simplified without data type analysis).
			IEnumerable<string> formatStrings = GetFormatstrings(referencedSrInterfaces).Where(x => x != null).Distinct();
		}

		private IEnumerable<string> GetAllSrInterfaceRefs(APPLICATION_SW_COMPONENT_TYPE component)
		{
			var prPorts = component.PORTS.Choice.OfType<PR_PORT_PROTOTYPE>().Where(x => x.PROVIDED_REQUIRED_INTERFACE_TREF.DEST == PORT_INTERFACE__SUBTYPES_ENUM.SENDER_RECEIVER_INTERFACE);
			var pPorts = component.PORTS.Choice.OfType<P_PORT_PROTOTYPE>().Where(x => x.PROVIDED_INTERFACE_TREF.DEST == PORT_INTERFACE__SUBTYPES_ENUM.SENDER_RECEIVER_INTERFACE);
			var rPorts = component.PORTS.Choice.OfType<R_PORT_PROTOTYPE>().Where(x => x.REQUIRED_INTERFACE_TREF.DEST == PORT_INTERFACE__SUBTYPES_ENUM.SENDER_RECEIVER_INTERFACE);

			return prPorts.Select(x => x.PROVIDED_REQUIRED_INTERFACE_TREF.Value).Union(
				pPorts.Select(x => x.PROVIDED_INTERFACE_TREF.Value)).Union(
				rPorts.Select(x => x.REQUIRED_INTERFACE_TREF.Value));
		}

		private IEnumerable<string> GetFormatstrings(IEnumerable<SENDER_RECEIVER_INTERFACE> interfaces)
		{
			foreach (SENDER_RECEIVER_INTERFACE @interface in interfaces)
			{
				foreach (VARIABLE_DATA_PROTOTYPE variable in @interface.DATA_ELEMENTS.VARIABLE_DATA_PROTOTYPE)
				{
					yield return variable.SW_DATA_DEF_PROPS?.SW_DATA_DEF_PROPS_VARIANTS?.SW_DATA_DEF_PROPS_CONDITIONAL?.Single().DISPLAY_FORMAT?.Value;
				}
			}
		}

		private static string GetNameFromDataRef(string reference)
		{
			if (reference.StartsWith(DataPath))
			{
				return reference.Substring(DataPath.Length);
			}
			else
			{
				return reference;
			}
		}
	}
}
